@@ -1,7 +1,7 @@
 /*
 
 queen-beecon.c - Queen Beecon Widget
-http://talk.maemo.org/showthread.php?t=
+http://talk.maemo.org/showthread.php?t=45388
 
 Copyright (c) 2010 No!No!No!Yes! (Alessandro Peralma)
 
@@ -92,6 +92,7 @@ gchar * p_fonts_types[] = {//AP1
 
 void queen_beecon_write_settings (QueenBeecon *self, gboolean newAll, gboolean newInstance, gboolean removeInstance);
 gboolean queen_beecon_update_content (QueenBeecon *self);
+static gboolean queen_beecon_expose_event (GtkWidget *widget, GdkEventExpose *event);
 
 //Dialogs prototypes:
 void queen_beecon_edit_add_dialog ( QueenBeecon *self, gboolean new, gint curr);
@@ -812,7 +813,8 @@ g_warning ("(%p) queen_beecon_connection_event",self);
 	{
 		self->priv->isConnected = FALSE;
 	}
-	queen_beecon_update_content(self);
+	if(self->priv->updNeworkPolicy != NETWORK__UNRELATED)
+		queen_beecon_update_content(self);
 	return TRUE;
 }
 
@@ -894,6 +896,106 @@ g_warning("(%p) queen_beecon_resize_hack",self);
 	return FALSE;
 }
 
+gboolean queen_beecon_update_content (QueenBeecon *self)
+{
+g_warning ("(%p) queen_beecon_update_content",self);
+	GdkColor color;
+
+	if(self->priv->widgetID == NULL)
+	{
+g_warning("(%p) Widget instance unknown... aborting",self);
+		return(FALSE);
+	}
+
+    //Font Mod//AP1v
+	gchar cmdTtl[256];//AP1
+	gchar resTtl[256];//AP1
+
+	PangoFontDescription *fd;//AP1
+	g_sprintf(cmdTtl,"%s %s %ld", p_fonts[self->priv->cmdFontName/4], p_fonts_types[self->priv->cmdFontName%4], (long) self->priv->cmdFontSize);//AP1
+	fd = pango_font_description_from_string(cmdTtl);//AP1
+	gtk_widget_modify_font(self->priv->cmdTitle_lb, fd);//AP1
+g_warning("(%p) Title Font: %s",self, cmdTtl);
+
+	g_sprintf(resTtl,"%s %s %ld", p_fonts[self->priv->resFontName/4], p_fonts_types[self->priv->resFontName%4], (long) self->priv->resFontSize);//AP1
+	fd = pango_font_description_from_string(resTtl);//AP1
+	gtk_widget_modify_font(self->priv->cmdResult_lb, fd);//AP1
+g_warning("(%p) Result Font: %s",self, resTtl);
+	//Font Mod//AP1^
+
+
+
+	if(self->priv->instanceCmd == NULL || self->priv->instanceTitle == NULL)
+	{
+		gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), "Error:");
+		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), "No commands");
+		return 0;
+	}
+
+	if (self->priv->cmdVisibilityPosition==0)//AP2
+		gtk_widget_hide (self->priv->cmdTitle_lb);//AP3
+	else
+		gtk_widget_show (self->priv->cmdTitle_lb);//AP3
+	if (self->priv->resVisibilityPosition==0)//AP2
+		gtk_widget_hide (self->priv->cmdResult_lb);//AP3
+	else
+		gtk_widget_show (self->priv->cmdResult_lb);//AP3
+
+g_warning ("(%p) Absolute Alignment cX=%d cY=%d rX=%d rY=%d", self,	self->priv->cmdVisibilityPosition, self->priv->cmdVisibilityPosition, self->priv->resVisibilityPosition, self->priv->resVisibilityPosition);
+g_warning ("(%p) Relative Alignment cX=%f cY=%f rX=%f rY=%f", self,	((self->priv->cmdVisibilityPosition-1)%3)*0.5, ((self->priv->cmdVisibilityPosition-1)/3)*0.5, ((self->priv->resVisibilityPosition-1)%3)*0.5, ((self->priv->resVisibilityPosition-1)/3)*0.5);
+g_warning ("(%p) Justification cmd=%d res=%d",self, self->priv->cmdJustify, self->priv->resJustify);
+
+	gdk_color_parse(p_colors[self->priv->cmdFgRGB].colorRGB,&color);
+	gtk_widget_modify_fg (self->priv->cmdTitle_lb, GTK_STATE_NORMAL, &color);
+
+	gdk_color_parse(p_colors[self->priv->resFgRGB].colorRGB,&color);
+	gtk_widget_modify_fg (self->priv->cmdResult_lb, GTK_STATE_NORMAL, &color);
+	gtk_misc_set_alignment(GTK_MISC (self->priv->cmdTitle_lb),((self->priv->cmdVisibilityPosition-1)%3)*0.5,((self->priv->cmdVisibilityPosition-1)/3)*0.5);
+	gtk_misc_set_alignment(GTK_MISC (self->priv->cmdResult_lb),((self->priv->resVisibilityPosition-1)%3)*0.5,((self->priv->resVisibilityPosition-1)/3)*0.5);
+	gtk_label_set_justify(GTK_LABEL(self->priv->cmdTitle_lb), self->priv->cmdJustify);
+	gtk_label_set_justify(GTK_LABEL(self->priv->cmdResult_lb), self->priv->resJustify);
+
+	gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), self->priv->instanceTitle);
+
+	if(self->priv->updNeworkPolicy == NETWORK__ONLY_CONNECTED && self->priv->isConnected == FALSE)
+		return TRUE;
+
+	if(self->priv->updNeworkPolicy == NETWORK__ONLY_DISCONNECTED && self->priv->isConnected == TRUE)
+		return FALSE;
+
+	FILE *fp;
+	gchar line[4096]="";//AP Big Big command output buffer//AP
+	size_t l;//AP
+
+	gboolean found = FALSE;
+	fp = popen (self->priv->instanceCmd, "r");
+g_warning("(%p) popen(%s) returned 0x%08X", self, self->priv->instanceCmd, (guint)fp);//AP >> 8 & 0xFF);//AP
+
+	if ((l=fread (line, 1, sizeof line, fp))!=0) //to this "buffer-based" for multiline handling in widget //AP
+		line[l-1]='\000';//AP
+
+	gint exitCode=pclose(fp);
+g_warning("(%p) ppclose(fp) returned 0x%08X", self, exitCode);
+
+	if ((self->priv->cmdExitStatus=(exitCode >> 8 & 0xff)) > 0x70) {
+		gchar buff[4096]="";//AP output buffer//AP
+		self->priv->cmdExitStatus=-1;
+		g_sprintf(buff,"[!?%04x]\n%s", exitCode, line);
+		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), buff);//AP
+	} else
+		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), line);//AP
+
+	queen_beecon_expose_event(GTK_WIDGET (self), NULL);
+
+	if( (self->priv->updateTimerID == 0) && (self->priv->delaySeconds > 0))
+	{
+g_warning("(%p) re-arming timer, delaySeconds=%d", self, self->priv->delaySeconds);
+		self->priv->updateTimerID = g_timeout_add_seconds (self->priv->delaySeconds, (GSourceFunc)queen_beecon_update_content, self);
+	}
+
+	return found;
+}
+
 static void queen_beecon_dispose (GObject *object)
 {
 g_warning ("(%p) queen_beecon_dispose",object);
@@ -1107,106 +1209,6 @@ g_warning ("queen_beecon_new");
 	return g_object_new (QUEEN_BEECON_TYPE, NULL);
 }
 
-gboolean queen_beecon_update_content (QueenBeecon *self)
-{
-g_warning ("(%p) queen_beecon_update_content",self);
-	GdkColor color;
-
-	if(self->priv->widgetID == NULL)
-	{
-g_warning("(%p) Widget instance unknown... aborting",self);
-		return(FALSE);
-	}
-
-    //Font Mod//AP1v
-	gchar cmdTtl[256];//AP1
-	gchar resTtl[256];//AP1
-
-	PangoFontDescription *fd;//AP1
-	g_sprintf(cmdTtl,"%s %s %ld", p_fonts[self->priv->cmdFontName/4], p_fonts_types[self->priv->cmdFontName%4], (long) self->priv->cmdFontSize);//AP1
-	fd = pango_font_description_from_string(cmdTtl);//AP1
-	gtk_widget_modify_font(self->priv->cmdTitle_lb, fd);//AP1
-g_warning("(%p) Title Font: %s",self, cmdTtl);
-
-	g_sprintf(resTtl,"%s %s %ld", p_fonts[self->priv->resFontName/4], p_fonts_types[self->priv->resFontName%4], (long) self->priv->resFontSize);//AP1
-	fd = pango_font_description_from_string(resTtl);//AP1
-	gtk_widget_modify_font(self->priv->cmdResult_lb, fd);//AP1
-g_warning("(%p) Result Font: %s",self, resTtl);
-	//Font Mod//AP1^
-
-	if(self->priv->updNeworkPolicy == NETWORK__ONLY_CONNECTED && self->priv->isConnected == FALSE)
-		return TRUE;
-
-	if(self->priv->updNeworkPolicy == NETWORK__ONLY_DISCONNECTED && self->priv->isConnected == TRUE)
-		return FALSE;
-
-
-	FILE *fp;
-	gchar line[4096]="";//AP Big Big command output buffer//AP
-	size_t l;//AP
-
-	gboolean found = FALSE;
-
-	if(self->priv->instanceCmd == NULL || self->priv->instanceTitle == NULL)
-	{
-		gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), "Error:");
-		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), "No commands");
-		return 0;
-	}
-
-	if (self->priv->cmdVisibilityPosition==0)//AP2
-		gtk_widget_hide (self->priv->cmdTitle_lb);//AP3
-	else
-		gtk_widget_show (self->priv->cmdTitle_lb);//AP3
-	if (self->priv->resVisibilityPosition==0)//AP2
-		gtk_widget_hide (self->priv->cmdResult_lb);//AP3
-	else
-		gtk_widget_show (self->priv->cmdResult_lb);//AP3
-
-g_warning ("(%p) Absolute Alignment cX=%d cY=%d rX=%d rY=%d", self,	self->priv->cmdVisibilityPosition, self->priv->cmdVisibilityPosition, self->priv->resVisibilityPosition, self->priv->resVisibilityPosition);
-g_warning ("(%p) Relative Alignment cX=%f cY=%f rX=%f rY=%f", self,	((self->priv->cmdVisibilityPosition-1)%3)*0.5, ((self->priv->cmdVisibilityPosition-1)/3)*0.5, ((self->priv->resVisibilityPosition-1)%3)*0.5, ((self->priv->resVisibilityPosition-1)/3)*0.5);
-g_warning ("(%p) Justification cmd=%d res=%d",self, self->priv->cmdJustify, self->priv->resJustify);
-
-	gdk_color_parse(p_colors[self->priv->cmdFgRGB].colorRGB,&color);
-	gtk_widget_modify_fg (self->priv->cmdTitle_lb, GTK_STATE_NORMAL, &color);
-
-	gdk_color_parse(p_colors[self->priv->resFgRGB].colorRGB,&color);
-	gtk_widget_modify_fg (self->priv->cmdResult_lb, GTK_STATE_NORMAL, &color);
-	gtk_misc_set_alignment(GTK_MISC (self->priv->cmdTitle_lb),((self->priv->cmdVisibilityPosition-1)%3)*0.5,((self->priv->cmdVisibilityPosition-1)/3)*0.5);
-	gtk_misc_set_alignment(GTK_MISC (self->priv->cmdResult_lb),((self->priv->resVisibilityPosition-1)%3)*0.5,((self->priv->resVisibilityPosition-1)/3)*0.5);
-	gtk_label_set_justify(GTK_LABEL(self->priv->cmdTitle_lb), self->priv->cmdJustify);
-	gtk_label_set_justify(GTK_LABEL(self->priv->cmdResult_lb), self->priv->resJustify);
-
-	gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), self->priv->instanceTitle);
-
-	fp = popen (self->priv->instanceCmd, "r");
-g_warning("(%p) popen(%s) returned 0x%08X", self, self->priv->instanceCmd, (guint)fp);//AP >> 8 & 0xFF);//AP
-
-	if ((l=fread (line, 1, sizeof line, fp))!=0) {//to this "buffer-based" for multiline handling in widget //AP
-		line[l-1]='\000';//AP
-		found = TRUE;
-	}
-
-	gint exitCode=pclose(fp);
-g_warning("(%p) ppclose(fp) returned 0x%08X", self, exitCode);
-
-	if ((self->priv->cmdExitStatus=(exitCode >> 8 & 0xff)) > 0x70) {
-		self->priv->cmdExitStatus=-1;
-		g_sprintf(line,"!?%08x",exitCode);
-	}
-
-	gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), line);//AP
-
-	queen_beecon_expose_event(GTK_WIDGET (self), NULL);
-
-	if( (self->priv->updateTimerID == 0) && (self->priv->delaySeconds > 0))
-	{
-g_warning("(%p) re-arming timer, delaySeconds=%d", self, self->priv->delaySeconds);
-		self->priv->updateTimerID = g_timeout_add_seconds (self->priv->delaySeconds, (GSourceFunc)queen_beecon_update_content, self);
-	}
-
-	return found;
-}
 
 //Separate window dialogs
 void queen_beecon_settings (HDHomePluginItem *hitem, QueenBeecon *self)
@@ -1240,6 +1242,11 @@ g_warning ("(%p) Window Position Current x=%d y=%d",self, self->priv->widXOrigin
 
 		GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
 
+
+		GtkWidget *cmdSelectionLabel = gtk_label_new ("");
+		gtk_label_set_markup (GTK_LABEL (cmdSelectionLabel), "<small>Command Selection:</small>");
+		gtk_container_add (GTK_CONTAINER (content_area), cmdSelectionLabel);
+
 		//BEGIN Command Selector
 
 		GtkWidget *selector = hildon_touch_selector_new_text ();
@@ -1250,11 +1257,6 @@ g_warning ("(%p) Window Position Current x=%d y=%d",self, self->priv->widXOrigin
 				if(g_strcmp0(self->priv->c_commands[i],self->priv->instanceCmd) == 0)
 					hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, i);
 		}
-
-		GtkWidget *cmdSelectionLabel = gtk_label_new ("");
-		gtk_label_set_markup (GTK_LABEL (cmdSelectionLabel), "<small>Command Selection:</small>");
-		gtk_container_add (GTK_CONTAINER (content_area), cmdSelectionLabel);
-
 		GtkWidget * cmdSelector = hildon_picker_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
 		hildon_button_set_title (HILDON_BUTTON (cmdSelector), "Commands: ");
 		hildon_button_add_title_size_group (HILDON_BUTTON (cmdSelector), group);
@@ -1629,14 +1631,15 @@ g_warning ("(%p) Window Position New x=%d y=%d",self, self->priv->widXOrigin, se
 
 
 		GtkWidget *updIntervalLabel = gtk_label_new ("");
-		gtk_label_set_markup (GTK_LABEL (updIntervalLabel), "<small>'0' disables periodic updating</small>");
+		gtk_label_set_markup (GTK_LABEL (updIntervalLabel), "<small>'0' disables periodic updating.</small>");
+		gtk_misc_set_alignment (GTK_MISC (updIntervalLabel), 0, 0);
 		gtk_container_add (GTK_CONTAINER (content_area), updIntervalLabel);
 
 
 		GtkWidget *networkPolicySel = hildon_touch_selector_new_text ();
-		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (networkPolicySel), "Doesn't Matter");
-		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (networkPolicySel), "Only when connected");
-		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (networkPolicySel), "Only when disconnected");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (networkPolicySel), "Disabled (Doesn't Matter)");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (networkPolicySel), "When connected");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (networkPolicySel), "When disconnected");
 		hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (networkPolicySel), 0, self->priv->updNeworkPolicy);
 
 		GtkWidget * networkPolBt = hildon_picker_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
@@ -1645,6 +1648,14 @@ g_warning ("(%p) Window Position New x=%d y=%d",self, self->priv->widXOrigin, se
 		hildon_button_set_alignment (HILDON_BUTTON (networkPolBt), 0, 0.5, 0, 0);
 		hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (networkPolBt), HILDON_TOUCH_SELECTOR (networkPolicySel));
 		gtk_container_add (GTK_CONTAINER (content_area), networkPolBt);
+
+		GtkWidget *networkPolBtHelp = gtk_label_new ("");
+		gtk_label_set_markup (GTK_LABEL (networkPolBtHelp), "<small>If enabled, the selected network event will trigger the command.</small>");
+		gtk_misc_set_alignment (GTK_MISC (networkPolBtHelp), 0, 0);
+		gtk_label_set_line_wrap (GTK_LABEL (networkPolBtHelp), TRUE);
+		gtk_container_add (GTK_CONTAINER (content_area), networkPolBtHelp);
+
+
 		//END
 
 		gtk_widget_show_all (dialog);
@@ -1658,6 +1669,7 @@ g_warning ("(%p) Window Position New x=%d y=%d",self, self->priv->widXOrigin, se
 		if (setDialogReturn == GTK_RESPONSE_ACCEPT)
 		{
 			int cmdSel = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (selector), 0);
+		    g_warning ("(%p) GTK_RESPONSE_ACCEPT cmdSel = %d c_size = %d",self, cmdSel, self->priv->c_size);
 			if(cmdSel < self->priv->c_size && cmdSel >= 0)
 			{
 				self->priv->instanceTitle = g_strdup(self->priv->c_titles[cmdSel]);
@@ -1682,6 +1694,15 @@ g_warning ("(%p) Moving Window Position x=%d y=%d",self, self->priv->widXOrigin,
 			else//AP3
 				self->priv->widHeight = self->priv->widWidth / SIZE_BEECON_RATIO;//Keep height aspect ration for beecon//AP3
 
+			GdkGeometry hints;
+
+			hints.min_width = (int)(self->priv->widWidth);//AP3
+			hints.min_height = (int)(self->priv->widHeight);//AP3
+			hints.max_width = (int)(SIZE_WIDTH_ALL);//AP3
+			hints.max_height = (int)(SIZE_HEIGHT_ALL);//AP3
+
+			gtk_window_set_geometry_hints (GTK_WINDOW (self), self->priv->homeWidget, &hints, GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
+
 			gtk_widget_set_size_request (GTK_WIDGET (self), (int)(self->priv->widWidth), (int)(self->priv->widHeight));//AP
 			gtk_window_resize (GTK_WINDOW (self), (int)(self->priv->widWidth), (int)(self->priv->widHeight));//AP
 			gtk_widget_set_size_request(self->priv->cmdTitle_lb, self->priv->widWidth, self->priv->widHeight);
@@ -1713,7 +1734,6 @@ g_warning ("(%p) Moving Window Position x=%d y=%d",self, self->priv->widXOrigin,
 				g_source_remove (self->priv->updateTimerID);
 				self->priv->updateTimerID = 0;
 			}
-			else
 
 			queen_beecon_write_settings (self, FALSE, FALSE, FALSE);
 			queen_beecon_update_content (self);
@@ -1726,7 +1746,9 @@ g_warning ("(%p) Moving Window Position x=%d y=%d",self, self->priv->widXOrigin,
 		}
 		else if(setDialogReturn == NON_GTK_RESPONSE_EDIT_CMD)
 		{
+
 			int cmdSel = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (selector), 0);
+		    g_warning ("(%p) NON_GTK_RESPONSE_EDIT_CMD cmdSel = %d c_size = %d",self, cmdSel, self->priv->c_size);
 			if(cmdSel < self->priv->c_size && cmdSel >= 0)
 			{
 				queen_beecon_edit_add_dialog(self, FALSE, cmdSel);
@@ -1802,9 +1824,15 @@ g_warning ("(%p) queen_beecon_edit_add_dialog new=%d curr=%d", self, new, curr);
 
 	gtk_widget_show_all (dialog);
 	int dialogRunResponse = gtk_dialog_run (GTK_DIALOG (dialog));
+
 	switch(dialogRunResponse)
 	{
 		case GTK_RESPONSE_ACCEPT:
+			if(g_strcmp0((gchar *)"",hildon_entry_get_text (HILDON_ENTRY (titleEntry))) == 0)
+			{
+				hildon_entry_set_text(HILDON_ENTRY (titleEntry), "[No Title]");
+			}
+
 			if(new)
 			{
 				AddCommand(self, g_strdup(hildon_entry_get_text (HILDON_ENTRY (titleEntry))), g_strdup(hildon_entry_get_text (HILDON_ENTRY (commandEntry))));
@@ -1873,6 +1901,12 @@ g_warning ("(%p) EditCommand title=[%s] command=[%s] index=[%d]", self, s_title,
 
 	self->priv->c_titles[index] = s_title;
 	self->priv->c_commands[index] = s_command;
+//vvvv BUG try to fix bug when NULL selector when back from command edit
+	g_free(self->priv->instanceTitle);
+	g_free(self->priv->instanceCmd);
+	self->priv->instanceTitle = g_strdup(s_title);
+	self->priv->instanceCmd = g_strdup(s_command);
+//^^^^ BUG try to fix bug when NULL selector when back from command edit
 }
 
 void DelCommand(QueenBeecon *self, int index)
